indexing

	description: "Base class for all Standard C or POSIX classes."

	author: "Berend de Boer"
	date: "$Date: 2003/10/02 $"
	revision: "$Revision: #6 $"

class

	STDC_BASE

inherit

	ANY

	STDC_CONSTANTS
		export
			{NONE} all
		end

	EPX_EXTERNAL_HELPER
		export
			{NONE} all
		end

	STDC_SECURITY_ACCESSOR


feature -- errno

	errno: STDC_ERRNO is
			-- Access to the variable that contains the error that occurred.
		once
			create Result
		ensure
			errno_not_void: Result /= Void
		end


feature {NONE} -- exceptions

	exceptions: expanded EXCEPTIONS
			-- Access to developer raised exceptions.

	do_raise (name: STRING) is
		do
			exceptions.raise (name)
		end

	raise_posix_error is
			-- Throws an exception when something that is considered
			-- fatal has occurred.
			-- Exception throwing can be disabled by calling
			-- `security.error_handling.disable_exceptions'
		require
			-- errno_should_be_set: errno.is_not_ok
		do
			if security.error_handling.exceptions_enabled then
				-- Unfreeze strings first because with exception we might
				-- not hit the unfreeze call in the caller.
				sh.unfreeze_all
				do_raise (errno.message)
			else
				if errno.first_value = 0 then
					errno.set_first
				end
			end
		end

	safe_call (res: INTEGER) is
			-- Raise an exception when value = -1.
		do
			if res = -1 then
				raise_posix_error
			end
		end


feature {NONE} -- test or set bits in an integer

	flip_bits (bit_field, bits: INTEGER; on: BOOLEAN): INTEGER is
			-- Change zero or more bits and return the result.
		do
			if on then
				Result := posix_or (bit_field, bits)
			else
				Result := posix_and (bit_field, posix_not (bits))
			end
		ensure
			bits_flipped: (not on) or test_bits (Result, bits)
		end

	test_bits (bit_field, bits: INTEGER): BOOLEAN is
			-- Do all `bits' in `bit_field' have the value 1?
		do
			Result := posix_and (bit_field, bits) = bits
		end


feature {NONE} -- Make bit operations on integers possible

	posix_and (op1, op2: INTEGER): INTEGER is
		external "C"
		end

	posix_not (op: INTEGER): INTEGER is
		external "C"
		end

	posix_or (op1, op2: INTEGER): INTEGER is
		external "C"
		end


end
