indexing

	description: "Class that covers current process related routines."
	usage: "Just inherit from this class."

	author: "Berend de Boer"
	date: "$Date: 2003/10/02 $"
	revision: "$Revision: #6 $"

class

	STDC_CURRENT_PROCESS


inherit

	STDC_PROCESS

	CAPI_STDLIB
		export
			{NONE} all
		end

	CAPI_LOCALE
		export
			{NONE} all
		end

	CAPI_TIME
		export
			{NONE} all
		end


feature -- My standard input/output/error

	stdin: STDC_TEXT_FILE is
		once
			create Result.attach_to_stream (stream_stdin, "r")
		ensure
			stdin_not_void: Result /= Void
			open_read: Result.is_open_read
		end

	stdout: STDC_TEXT_FILE is
		once
			create Result.attach_to_stream (stream_stdout, "w")
		ensure
			stdout_not_void: Result /= Void
			open_read: Result.is_open_write
		end

	stderr: STDC_TEXT_FILE is
		once
			create Result.attach_to_stream (stream_stderr, "w")
		ensure
			stderr_not_void: Result /= Void
			open_read: Result.is_open_write
		end


feature {NONE} -- ways to exit a program

	abort is
			-- Causes abnormal process termination.
			-- On many systems, dumps core.
		do
			posix_abort
		ensure
			we_should_never_come_here: False
		end


	exit (a_status: INTEGER) is
			-- Terminate this process with exit code `a_status'.
		do
			posix_exit (a_status)
		end

	exit_with_failure is
			-- Exit program immediately with OS dependent failure code.
		do
			exit (EXIT_FAILURE)
		end

	exit_with_success is
			-- Exit program immediately with OS dependent OK code.
		do
			exit (EXIT_SUCCESS)
		end


feature -- various

	clock: INTEGER is
			-- return approximation of processor time used by the
			-- program, or -1 if unknown
		do
			Result := posix_clock
		end


feature -- Random numbers

	random: INTEGER is
			-- Returns a pseudo-random integer between 0 and `RAND_MAX'.
		do
			Result := posix_rand
		ensure
			random_in_range: random >= 0 and random <= RAND_MAX
		end

	rand: INTEGER is
		obsolete "use random instead."
		do
			Result := random
		end

	set_random_seed (a_seed: INTEGER) is
			-- Sets `a_seed' as the seed for a new sequence of
			-- pseudo-random integers to be returned by `random'. These
			-- sequences are repeatable by calling `set_random_seed' with
			-- the same seed value. If no seed value is provided, the
			-- `random' function is automatically seeded with a value of
			-- 1.
		require
			seed_not_negative: a_seed >= 0
		do
			posix_srand (a_seed)
		end

	srandom (a_seed: INTEGER) is
		obsolete "Use set_random_seed instead."
		do
			set_random_seed (a_seed)
		end


feature {NONE} -- global locale

	locale: STRING is
			-- return current locale
		do
			Result := sh.pointer_to_string (posix_setlocale (LC_ALL, default_pointer))
		end

	numeric_format: STDC_LOCALE_NUMERIC is
			-- various information for formatting numbers and monetary
			-- quantities
		once
			create Result
		end

	set_locale (category: INTEGER; new_locale: STRING) is
			-- set given locale to `new_locale'
		local
			p: POINTER
		do
			p := posix_setlocale (category, sh.string_to_pointer (new_locale))
			if p = default_pointer then
				raise_posix_error
			end
			sh.unfreeze_all
		end

	set_c_locale is
			-- set locale to the Standard C locale (the default)
		do
			set_locale (LC_ALL, "C")
		end

	set_native_decimal_point is
			-- set the decimal point character
		do
			set_locale (LC_NUMERIC, "")
		end

	set_native_locale is
			-- set entire locale to the native's setting
		do
			set_locale (LC_ALL, "")
		end

	set_native_time is
			-- set time display to the native's setting
		do
			set_locale (LC_TIME, "")
		end


end
